/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_core_cookieparser
#define headerfile_winix_core_cookieparser

#include <fcgiapp.h>
#include "httpsimpleparser.h"
#include "requesttypes.h"
#include "log.h"



class CookieParser : public HttpSimpleParser
{

	const char * cookie_string;
	CookieTab * cookie_tab;
	

protected:


	virtual int GetChar()
	{
		if( !cookie_string || *cookie_string == 0 )
			return -1;
		
		return (int)(unsigned char)*(cookie_string++);	
	}
	

	virtual void Parameter(std::string & name, std::string & value)
	{
		// Cookie names are case insensitive according to section 3.1 of RFC 2965
		ToLower(name);
		
		std::pair<CookieTab::iterator, bool> res = cookie_tab->insert( std::make_pair(name, value) );	
		
		log << log2 << "Cookie, name: \"" << name << "\", value: \"" << value << "\"";
		
		if( res.second == false )
		{
			res.first->second = value;
			log << " (overwritten)";
		}
			
		log << logend;
	}


public:


	CookieParser()
	{
		HttpSimpleParser::separator               = ';';
		HttpSimpleParser::value_can_be_quoted     = true;
		HttpSimpleParser::skip_white_chars        = true;
		HttpSimpleParser::recognize_special_chars = false;
	}


	// cookie_string can be null
	void Parse(const char * cookie_string_, CookieTab & cookie_tab_)
	{
		cookie_string = cookie_string_;
		cookie_tab = &cookie_tab_;

		HttpSimpleParser::Parse();
	}

};



#endif
