/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010-2011, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_core_mountparser
#define headerfile_winix_core_mountparser

#include <map>
#include <string>
#include <vector>
#include <stdlib.h>
#include <limits.h>

#include "mount.h"
#include "item.h"
#include "dirs.h"



class MountParser
{
public:

	MountParser();

	void SkipStaticDirs(bool skip);
	void SetStaticMountId(int id);

	void Parse(const std::wstring & input, std::map<long, Mount> & output);
	void SetDirs(Dirs * pdirs);
	void SetMountTypeTab(const std::vector<std::wstring> & tab);
	void SetMountFsTab(const std::vector<std::wstring> & tab);
	void SetMountParTab(const std::vector<std::wstring> & tab);

private:

	Dirs * dirs;
	bool skip_static;
	int static_mount_id;

	const std::vector<std::wstring> * mount_type_tab;
	const std::vector<std::wstring> * mount_fs_tab;
	const std::vector<std::wstring> * mount_par_tab;

	bool IsWhite(int c);
	void SkipWhite();
	void SkipLine();
	void ReadWordQuote(std::wstring & res);
	void ReadWordWhite(std::wstring & res);
	void ReadWordComma(std::wstring & res);
	void ReadWord(std::wstring & res, bool comma_bracket_separator = false);
	void ReadParamArgsLoop(Mount::ParamRow::ParamArg & args);
	void ReadParamArgs(Mount::ParamRow::ParamArg & args);
	void ReadParamName(std::wstring & res);
	void ReadParam(std::wstring & res, Mount::ParamRow::ParamArg & args);
	int FindIndex(const std::vector<std::wstring> * tab, const std::wstring & value);
	bool ReadMountType();
	bool ReadMountPoint();
	bool ReadFs();
	void LogMountParams();
	void ReadMountParams();
	void ReadRow();
	void AddParams(Mount::Param & src, Mount::Param & dst);
	bool AddParamsBefore(long dir_id);
	void AddParamsBefore();
	void AddParamsAfter();

	const wchar_t * pinput;
	std::wstring temp;
	std::wstring last_dir;
	std::wstring temp_arg;
	Mount::ParamRow::ParamArg param_args;
	Mount mount;
	Item * pdir;
	std::map<long, Mount> * poutput;
	std::pair<std::map<long, Mount>::iterator, bool> mount_inserted;
};


#endif
