/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_core_sessioncontainer
#define headerfile_winix_core_sessioncontainer

#include <list>
#include <map>
#include <ctime>

#include "session.h"
#include "cur.h"
#include "config.h"



class SessionContainer
{
public:

	typedef std::list<Session>       Table;
	typedef Table::iterator          Iterator;
	typedef std::map<long, Iterator> IndexId;
	typedef IndexId::iterator        IdIterator;


	SessionContainer();

	void SetCur(Cur * pcur);
	void SetConfig(Config * pconfig);
	void SetTmpSession(Session * psession);

	void Clear();

	size_t    Size();
	Iterator  Begin();
	Iterator  End();
	Session & Back();
	Iterator AddSession(long id);
	Iterator FindById(long);

	IdIterator IdBegin();
	IdIterator IdEnd();

	void EraseById(long id);

	bool ChangeSessionId(Iterator ses, long new_id);
	bool ChangeSessionId(long old_id, long new_id);

private:

	Table table;
	IndexId index_id;

	Cur     * cur;
	Config  * config;
	Session * tmp_session;

	Session empty_session;

	// in FreeBSD implementation (GCC) list::size() has linear complexity
	// so we use our own table_size with O(1)
	size_t table_size;

};


#endif
