/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "emacs.h"
#include "templates/templates.h"
#include "functions.h"



namespace Fun
{

Emacs::Emacs()
{
	fun.url = L"emacs";
}



bool Emacs::HasAccess(const Item & item)
{
	if( cur->session->puser && cur->session->puser->super_user )
		// super user can use emacs everywhere
		return true;

	if( !system->HasWriteAccess(item) )
		return false;

	if( !system->mounts.pmount->IsPar(system->mounts.MountParEmacsOn()) )
		return true;

	if( system->mounts.pmount->IsArg(system->mounts.MountParEmacsOn(), cur->request->dir_tab.size()) )
		return true;	
	
return false;
}



bool Emacs::HasAccess()
{
	return HasAccess(*cur->request->last_item);
}





bool Emacs::PostEmacsCheckAbuse(bool adding)
{
	if( !system->rebus.CheckRebus() )
	{
		cur->request->status = WINIX_ERR_INCORRECT_REBUS;
		return false;
	}

	// !! is tested in createthread once
	functions->CheckGetPostTimes();

	if( cur->session->spam_score > 0 )
	{
		cur->request->status = WINIX_ERR_SPAM;
		log << log1 << "Emacs: ignoring due to suspected spamming" << logend;
		return false;
	}

return true;
}


// !! zmienic nazwy 
// albo w ogole te metody nie sa potrzebne teraz (byly zmiany)
void Emacs::PostFunEmacsModifyMountPoint(bool adding)
{
	system->RedirectTo(cur->request->item);
}




int Emacs::NotifyCodeEdit()
{
// !! nie potrzebne
//	if( system->mounts.pmount->type == system->mounts.MountTypeThread() )
//		return WINIX_NOTIFY_CODE_THREAD_POST_CHANGED;

return WINIX_NOTIFY_CODE_EDIT;
}




int Emacs::NotifyCodeAdd()
{
// !! nie potrzebne
//	if( system->mounts.pmount->type == system->mounts.MountTypeThread() )
//		return WINIX_NOTIFY_CODE_THREAD_REPLAYED;

return WINIX_NOTIFY_CODE_ADD;
}




void Emacs::MakePost()
{
	bool adding = !cur->request->is_item;

	if( !adding )
		old_url = cur->request->item.url;

	functions->ReadItem(cur->request->item, Item::file);

	if( adding )
		functions->SetUser(cur->request->item); // set user before checking the rebus

	if( !PostEmacsCheckAbuse(adding) )
		return;

	if( adding )
	{
		cur->request->is_item          = true;	// !! moze lepiej nie ustawiac is_item? (bo jak wystapi blad np dodania do bazy danych
												//    to formularz edycji zmieni sie z 'dodaj' na 'edytuj'
		cur->request->item.privileges  = system->NewFilePrivileges();
		cur->request->status = system->AddFile(cur->request->item, NotifyCodeAdd());
	}
	else
	{
		cur->request->status = system->EditFile(cur->request->item, cur->request->item.url != old_url, NotifyCodeEdit());
	}

	if( cur->request->status == WINIX_ERR_OK )
	{
		PostFunEmacsModifyMountPoint(adding);
		functions->CheckSpecialFile(cur->request->item);
	}
	else
	{
		log << log1 << "Emacs: error: "	<< cur->request->status << logend;
	}
}








} // namespace
