/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_functions_mv
#define headerfile_winix_functions_mv

#include "functionbase.h"




namespace Fun
{



class Mv : public FunctionBase
{
public:

	Mv();
	bool HasAccess();
	void MakePost();

	// moving a directory
	// new_url can be empty (in such a case the old one is preserved)
	// src_dir will be accordingly updated
	bool MoveDir(Item & src_dir, long dst_dir_id, const std::wstring & new_url, bool check_access = true);
	bool MoveDir(Item & src_dir, const std::wstring & dst_path, bool check_access = true);

	// moving only the content of src_dir
	// dst_path should point to an existing directory
	// src_dir will be accordingly updated
	void MoveDirContent(Item & src_dir, long dst_dir_id, bool check_access = true);
	void MoveDirContent(Item & src_dir, const std::wstring & dst_dir, bool check_access = true);

	// moving a file
	// new_url can be empty (the old one is used then)
	// src_file will be accordingly updated
	bool MoveFileOrSymlink(Item & src_file, long dst_dir_id, const std::wstring & new_url, bool check_access = true);
	bool MoveFileOrSymlink(Item & src_file, const std::wstring & dst_path, bool check_access = true);


private:


	// directories parsed by ParseDir()
	std::vector<Item*> out_dir_tab;		// destination directories
	Item out_item;						// destination file/symlink (if out_has_file is true)
	bool out_has_file;					// if true then out_item is set
	std::wstring out_filename;			// the last part in an input path (not an existing directory, file or symlink)
										// can be empty

	// temporaries
	std::wstring out_path;
	std::wstring old_url;

	std::wstring old_static_path;
	std::wstring old_static_thumb_path;
	std::wstring new_static_path;
	std::wstring new_static_thumb_path;

	// for files/symlinks in a directory
	std::vector<Item> files_item_tab;
	DbItemQuery files_iq;

	// for moving content of a directory (all dirs/files/symlinks)
	DbItemQuery content_dir_iq;
	std::vector<Item> item_tab;


	bool CheckAccessFromToDir(const Item & dir, bool only_content);
	bool CanRemoveRenameChild(const Item & child);
	void Prepare();
	bool ParseDirCheckLastName();
	bool ParseDir(const std::wstring & dst_path, bool check_access);
	bool MoveStaticFile(const std::wstring & from, const std::wstring & to);
	void MoveStaticFile(Item & item);
	void MoveFilesPrepareTreeGo(const Item & src_dir);
	void MoveFilesPrepareTree(const Item & src_dir);
	void MoveFilesTree(const Item & dir);
	bool MoveDir(Item & src_dir, std::vector<Item*> & dst_dir_tab, const std::wstring & dst_name);
	bool MoveFileOrSymlink(Item & src_file, std::vector<Item*> & dst_dir_tab, const std::wstring & new_url);
	void Clear();
	bool MoveDir2(Item & src_dir, long dst_dir_id, const std::wstring & new_url, bool check_access);
	bool MoveDir2(Item & src_dir, const std::wstring & dst_path, bool check_access);
	bool MoveFileOrSymlink2(Item & src_file, long dst_dir_id, const std::wstring & new_url, bool check_access);
	bool MoveFileOrSymlink2(Item & src_file, const std::wstring & dst_path, bool check_access);
	void MoveDirContent2(Item & src_dir, long dst_dir_id, bool check_access);
	void MoveAllFilesFromDir(Item & src_dir, std::vector<Item*> & dst_dir_tab, bool check_access);
	void MoveDirContent2(Item & src_dir, const std::wstring & dst_dir, bool check_access);


};


} // namespace

#endif
