/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_functions_rm
#define headerfile_winix_functions_rm

#include "functionbase.h"



namespace Fun
{


class Rm : public FunctionBase
{
public:

	Rm();
	bool HasAccess();
	void MakePost();

	// removing the whole directory structure
	// if remove_this_dir is false then only content of dir is removed
	void RemoveDirTree(Item & dir, bool remove_this_dir, bool check_access = true);

	// removing either a file or a symlink
	bool RemoveFileOrSymlink(Item & item, bool check_access = true);

	// removing either a directory or a symlink or a file
	// if item_id is a directory then the whole subdirectories are removed too
	bool RemoveItemById(long item_id, bool check_access = true);

	// removing either a directory or a symlink or a file
	// if path is a directory then the whole subdirectories are removed too
	// path must begin with a slash (or can be empty then the root directory is removed)
	bool RemoveItemByPath(const std::wstring & path, bool check_access = true);


private:

	// for deleting content in a directory (files and symlinks)
	DbItemQuery content_dir_iq;
	std::vector<Item> content_item_tab;
	std::wstring path;

	// for logging
	std::wstring old_url;

	// for removing an item by id
	DbItemQuery rm_by_id_iq;
	Item rm_by_id_item;

	std::vector<Item*> rm_path_dir_tab;
	Item rm_path_item;

	bool HasAccessToDir(const Item & dir, bool only_content);
	void Prepare();
	void Clear();

	bool RemoveFile(Item & item);
	bool RemoveDirFiles(long dir_id, bool check_access);
	void RemoveCurrentDir(Item * parent_dir, Item * current_dir, bool check_access);
	void RemoveDirTree(Item * parent_dir, Item * current_dir, bool remove_this_dir, bool check_access);
	void RemoveStaticFile(const std::wstring & path);
	void RemoveStaticFile(const Item & item);
	void RemoveDirContent();
	void RemoveDir();
	void CreateJSON(bool status);

};


} // namespace

#endif
