/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2011, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "sort.h"
#include "core/plugin.h"



namespace Fun
{

Sort::Sort()
{
	fun.url = L"sort";
}


bool Sort::HasAccess()
{
	if( cur->request->is_item )
		return system->HasWriteAccess(cur->request->item);

	// for directories we always return true
	// (permissions are checked later)
	return true;
}



void Sort::GetDirContent()
{
	iq.sel_content = false;
	iq.WhereParentId(cur->request->dir_tab.back()->id);
	db->GetItems(cur->request->item_tab, iq);
	system->CheckWriteAccessToItems(cur->request->item_tab);
}


bool Sort::SortHelper::operator()(size_t t1, size_t t2)
{
	return psort->cur->request->item_tab[t1].id < psort->cur->request->item_tab[t2].id;
}


bool Sort::SortFun2(const SortPair & s1, const SortPair & s2)
{
	return s1.id < s2.id;
}



void Sort::CreateItemTab()
{
	size_t len = cur->request->item_tab.size();
	item_tab.resize(len);

	for(size_t i=0 ; i<len ; ++i)
		item_tab[i] = i;

	std::sort(item_tab.begin() , item_tab.end(), SortHelper(this));
}



void Sort::CreateItemTab2()
{
PostTab::iterator i2 = cur->request->post_tab.begin();
SortPair sp;

	item_tab2.clear();
	item_tab2.reserve(cur->request->post_tab.size());

	// post_tab is sorted in lexicographical order but we should sort it by numbers

	for( ; i2 != cur->request->post_tab.end() ; ++i2 )
	{
		if( IsSubStringNoCase(L"sort", i2->first.c_str()) )
		{
			sp.id = Tol(i2->first.c_str() + 4);
			sp.sort_index = Toi(i2->second);

			item_tab2.push_back(sp);
		}
	}

	std::sort(item_tab2.begin(), item_tab2.end(), SortFun2);
}


void Sort::UpdateSortIndex(Item & item, int sort_index)
{
	item.sort_index = sort_index;

	if( db->EditSortIndexItemById(item.id, sort_index) == WINIX_ERR_OK )
	{
		log << log2
			<< "Sort: updated sort index, item_id=" << item.id
			<< ", url=" << item.url
			<< ", sort_index=" << sort_index << logend;
	}
}



void Sort::UpdateSortIndexes()
{
size_t i1 = 0;
size_t i2 = 0;

	while( i1 < item_tab.size() && i2 < item_tab2.size() )
	{
		long id1 = cur->request->item_tab[item_tab[i1]].id;
		long id2 = item_tab2[i2].id;

		if( id1 == id2 )
		{
			int sort_index = item_tab2[i2].sort_index;
			Item & item    = cur->request->item_tab[item_tab[i1]];

			if( system->HasWriteAccess(item) )
				UpdateSortIndex(item, sort_index);

			++i1;
			++i2;
		}
		else
		if( id1 < id2 )
		{
			++i1;
		}
		else
		{
			++i2;
		}
	}
}



void Sort::MakePost()
{
	if( cur->request->is_item )
	{
		int sort_index = Toi(cur->request->PostVar(L"sortindex"));
		UpdateSortIndex(cur->request->item, sort_index);
	}
	else
	{
		GetDirContent();
		CreateItemTab();
		CreateItemTab2();
		UpdateSortIndexes();
	}

	plugin.Call(WINIX_DIR_CONTENT_SORTED, cur->request->dir_tab.back());

	system->RedirectToLastItem();
}


void Sort::MakeGet()
{
	if( !cur->request->is_item )
		GetDirContent();
}




} // namespace

