/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2011-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */

#ifndef headerfile_winix_plugins_groupitem_groups
#define headerfile_winix_plugins_groupitem_groups

#include <map>
#include <vector>
#include <string>
#include "space/spaceparser.h"



namespace GroupItem
{


class Groups
{
public:

	/*
		reading all space
		(sets and groups)
	*/
	PT::Space * GetSpace();


	/*
		building all indexes
	*/
	void Reindex();


	bool Find(const std::wstring & set, const std::wstring & value, size_t & seti, size_t & groupi);



	/*	
	*/
	size_t Size(size_t seti, size_t groupi);

	const std::wstring & GetOption(size_t seti, size_t groupi, size_t valuei, const wchar_t * option);
	const std::wstring & GetOption(size_t seti, size_t groupi, size_t valuei, const std::wstring & option);


	/*
		removing all groups
	*/
	void Clear();


private:

	/*
		our space:
		set1 (

		# group1
		(
			( value = "value1"
			  other = "foo" )

			( value = "something"
			  other = "x" )
		)

		# group2
		(

		)

		) # end of set1

		# we can have more sets
		# sets should have a unique name
		# groups don't have a name
	*/
	PT::Space space;


	/*
		indexes to a set of groups
	*/
	typedef std::map<std::wstring, size_t> SetIndex;
	SetIndex set_index;

	/*
		indexes to a groups in a set
		mapping: value->group index
	*/
	typedef std::map<std::wstring, size_t> GroupIndex;

	/*
		group indexes in a specified set
		std::vector<how many sets there are>
	*/
	std::vector<GroupIndex> group_index_tab;

	const std::wstring empty_str;


	std::wstring sort_by;
	bool sort_asc;
	PT::Space::Value sort_value;

	struct SortFunHelper
	{
		Groups * groups;
		SortFunHelper(Groups * pgroups) : groups(pgroups) {};
		size_t SortValue(const std::wstring & val);
		bool operator()(PT::Space * sp1, PT::Space * sp2);
	};



	void ReindexGroups(GroupIndex & group_index, PT::Space & set);
	void SortValues(PT::Space & group);
};

}

#endif

