/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include <algorithm>
#include "sessiondata.h"


namespace Ticket
{

SessionData::SessionData()
{
	fun_rm = 0;
}



SessionData::~SessionData()
{
	Clear();
}


void SessionData::Clear()
{
	RemoveFiles(create_space_map, false);
	RemoveFiles(edit_space_map,   true);
}



void SessionData::RemoveFiles(SessionData::SpaceMap & space_map, bool only_stale_files)
{
SpaceMap::iterator i;

	if( fun_rm )
	{
		for(i=space_map.begin() ; i!=space_map.end() ; ++i)
		{
			if( only_stale_files )
				RemoveAllStaleFiles(i->second.new_space, i->second.old_space);
			else
				RemoveAllFiles(i->second.new_space);
		}

		space_map.clear();
	}
}



void SessionData::RemoveAllFiles(PT::Space & new_space)
{
	std::vector<long> new_file_tab;
	BuildFileList(new_file_tab, new_space);

	for(size_t i=0 ; i<new_file_tab.size() ; ++i)
		fun_rm->RemoveItemById(new_file_tab[i], false);
}



void SessionData::RemoveAllStaleFiles(PT::Space & new_space, PT::Space & old_space)
{
	std::vector<long> new_file_tab;
	std::vector<long> old_file_tab;
	size_t n = 0;
	size_t o = 0;

	BuildFileList(new_file_tab, new_space);
	BuildFileList(old_file_tab, old_space);

	std::sort(new_file_tab.begin(), new_file_tab.end());
	std::sort(old_file_tab.begin(), old_file_tab.end());

	// removes only those items present on new_file_tab
	// but not existing on old_space_tab

	while( n < new_file_tab.size() && o < old_file_tab.size() )
	{
		if( new_file_tab[n] < old_file_tab[o] )
		{
			fun_rm->RemoveItemById(new_file_tab[n], false);
			n += 1;
		}
		else
		if( new_file_tab[n] == old_file_tab[o] )
		{
			n += 1;
			o += 1;
		}
		else
		{
			o += 1;
		}
	}

	while( n < new_file_tab.size() )
	{
		fun_rm->RemoveItemById(new_file_tab[n], false);
		n += 1;
	}
}





void SessionData::BuildFileList(std::vector<long> & file_tab, PT::Space & space)
{
	file_tab.clear();

	for(size_t i=0 ; i<space.spaces.size() ; ++i)
	{
		PT::Space & sp = *space.spaces[i];

		if( sp.name == L"param" )
			CheckFile(file_tab, sp);
	}
}


void SessionData::CheckFile(std::vector<long> & file_tab, PT::Space & space)
{
	for(size_t i=0 ; i<space.spaces.size() ; ++i)
	{
		PT::Space & subsp = *space.spaces[i];

		if( subsp.name == L"file" )
		{
			std::wstring * file_id_str = subsp.GetValue(L"itemid");

			if( file_id_str )
			{
				long file_id = Tol(*file_id_str);
				file_tab.push_back(file_id);
			}
		}
	}
}




Ticket & SessionData::GetTicket(long id, SessionData::TicketMap & ticket_map, bool * is_new)
{
	std::pair<TicketMap::iterator, bool> res = ticket_map.insert( std::make_pair(id, Ticket()) );

	if( is_new )
		*is_new = res.second;

return res.first->second;
}



PT::Space & SessionData::GetOldSpace(long id, SpaceMap & space_map, bool * is_new)
{
	std::pair<SpaceMap::iterator, bool> res = space_map.insert( std::make_pair(id, SpacePair()) );

	if( is_new )
		*is_new = res.second;

return res.first->second.old_space;
}


PT::Space & SessionData::GetNewSpace(long id, SpaceMap & space_map, bool * is_new)
{
	std::pair<SpaceMap::iterator, bool> res = space_map.insert( std::make_pair(id, SpacePair()) );

	if( is_new )
		*is_new = res.second;

return res.first->second.new_space;
}


} // namespace



