/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2011, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "templates.h"
#include "core/misc.h"


namespace TemplatesFunctions
{
static std::string urlencode_tmp;
static std::string qencode_tmp;


void fil_urlencode(Info & i)
{
	UrlEncode(i.in.Str(), urlencode_tmp);
	i.out << R(urlencode_tmp);
}


void fil_qencode(Info & i)
{
	QEncode(i.in.Str(), qencode_tmp);
	i.out << R(qencode_tmp);
}


void fil_capitalize(Info & i)
{
	const std::wstring & str = i.in.Str();

	for(size_t a=0 ; a<str.size() ; ++a)
		i.out << R(locale.ToCapital(str[a]));
}


void fil_tosmall(Info & i)
{
	const std::wstring & str = i.in.Str();

	for(size_t a=0 ; a<str.size() ; ++a)
		i.out << R(locale.ToSmall(str[a]));
}


// a first letter in a sentence will be capitalized
void fil_firstup(Info & i)
{
	bool was_dot = true;
	const std::wstring & str = i.in.Str();
	
	for(size_t a=0 ; a<str.size() ; ++a)
	{
		if( was_dot )
		{
			if( str[a]!=' ' && str[a]!='\t' && str[a]!=13 && str[a]!=10 && str[a]!=160 )
				was_dot = false;

			i.out << R(locale.ToCapital(str[a]));
		}
		else
		{
			i.out << R(str[a]);
		}

		if( str[a] == '.' )
			was_dot = true;
	}
}


// a first letter in each word will be capitalized
void fil_first_wordup(Info & i)
{
	bool was_white = true;
	const std::wstring & str = i.in.Str();
	
	for(size_t a=0 ; a<str.size() ; ++a)
	{
		if( was_white )
		{
			i.out << R(locale.ToCapital(str[a]));
		}
		else
		{
			i.out << R(str[a]);
		}

		was_white = (str[a]==' ' || str[a]=='\t' || str[a]==13 || str[a]==10 || str[a]==160);
	}
}


} // namespace
